/* mod_bugfinder - hunt for 1x1 images
   When we detect such a beast it is too late to do anything about it,
   so we just log the URI and assume a log-analyzer will blacklist them
*/

#include "squid.h"
#include "module.h"
#include "patfile.h"

#define MODNAME "mod_bugfinder"

#define HDRBUFSIZ 24
#define TRIG 3

classdef(private bugfinderFilterObject, filterObject, ({
    int posin;
    unsigned char bufr[HDRBUFSIZ];
}))

#define BUGFINDER_C
#include <classes.dh>
#include <classes.th>

Ddef_none(bugfinderFilterObject)

CBDATA_TYPE(bugfinderFilterObject);

static int cfFilter(filterObject *this, MemBuf *target,
                    const char *buf, int len)
{
    bugfinderFilterObject *cf = CC(this, bugfinderFilterObject);
    debug(93,9) (MODNAME ": %s: pos %d len %d\n", moduleUrlClean(cf->uri),
		 cf->posin, len);
    if (cf->posin >= 0 && cf->posin < HDRBUFSIZ) {
	memcpy(cf->bufr+cf->posin, buf, XMIN(len, HDRBUFSIZ-cf->posin));
	cf->posin += len;
    }
    if (cf->posin >= HDRBUFSIZ) {
	int x = 10000, y = 10000;
	if (!memcmp(cf->bufr, "GIF8", 4)) {
	    x = cf->bufr[6] + (cf->bufr[7]<<8);
	    y = cf->bufr[8] + (cf->bufr[9]<<8);
	} else if (!memcmp(cf->bufr, "\x89PNG", 4)) {
	    x = ntohl(*(unsigned long *)(cf->bufr+16));
	    y = ntohl(*(unsigned long *)(cf->bufr+20));
	}
	if (x < TRIG && y < TRIG) {
	    debug(93, 0) (MODNAME ": is a %dx%d image: %s\n", x, y,
			  moduleUrlClean(cf->uri));
	}
	cf->posin = -1;
    }
    memBufAppend(target, buf, len);
    return len;
}

/* moduleObject->filter for FIL_CONTFILTER:
   filterObject constructor.
*/
static void *bugfinderFilter(moduleObject *this, const void *arg)
{
    bugfinderFilterObject *f;
    const repParam *rp = arg;
    if (this->patFile && (patfileCheckReload(this->patFile),
                          patfileMatch(this->patFile, rp->uri)))
        return NULL;

    CBDATA_INIT_TYPE(bugfinderFilterObject);
    f = new(bugfinderFilterObject);
    f->filter = cfFilter;
    f->posin = 0;

    return f;
}

CBDATA_TYPE(moduleObject);

static void moduleInit1(const char *t, patFileObject *p)
{
    moduleObject *m;
    CBDATA_INIT_TYPE(moduleObject);
    m = new(moduleObject);
    m->chain.typ = FIL_CONTFILTER;
    m->info.version = MODULE_API_VERSION;
    m->description = "Web-bug logger";
    m->trigger = t;
    m->filter = bugfinderFilter;
    m->patFile = REF(p);

    moduleRegister(m);
}

void moduleInit(const wordlist *args)
{
    patFileObject *pf = args ? patfileNew(args->key, 0) : NULL;
    moduleInit1("image/gif", pf);
    moduleInit1("image/png", pf);
}

